/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "globus_gridftp_server_s3_base.h"

int
globus_i_s3_mime_types_parse(
    const char                         *mime_types)
{
    FILE                               *mtf;
    char                                linebuf[256];
    char                                mime_type[256];
    char                                extensions[256];
    int                                 rc;
    int                                 res = GLOBUS_SUCCESS;
    globus_i_s3_mime_t                 *mime_hashentry;
    char                               *p;

    rc = globus_hashtable_init(
        &globus_i_s3_mime_extensions,
        1549,
        globus_hashtable_string_hash,
        globus_hashtable_string_keyeq);
    if (rc != GLOBUS_SUCCESS)
    {
        res = rc;
        goto hashtable_init_fail;
    }
    if (mime_types == NULL)
    {
        goto done;
    }

    mtf = fopen(mime_types, "r");
    if (mtf == NULL)
    {
        res = GLOBUS_FAILURE;
        goto fopen_fail;
    }
    while ((p = fgets(linebuf, sizeof(linebuf), mtf)) != NULL)
    {
        char *c = strchr(p, '#');
        if (c)
        {
            *c = '\0';
        }
        rc = sscanf(p, "%s %[^\n]", mime_type, extensions);
        if (rc == 2)
        {
            char *estart = extensions;

            while (*estart)
            {
                size_t token_size = strcspn(estart, " \t\n\r");

                if (token_size > 0)
                {
                    if ((token_size + 1) < sizeof(mime_hashentry->extension))
                    {
                        mime_hashentry = malloc(sizeof(globus_i_s3_mime_t)
                                + strlen(mime_type) + 1);
                        if (mime_hashentry == NULL)
                        {
                            res = GLOBUS_FAILURE;
                            goto entry_alloc_fail;
                        }

                        memcpy(mime_hashentry->extension, estart, token_size);
                        mime_hashentry->extension[token_size] = 0;
                        strcpy(mime_hashentry->mime_type, mime_type);

                        rc = globus_hashtable_insert(
                            &globus_i_s3_mime_extensions,
                            mime_hashentry->extension,
                            mime_hashentry);
                        if (rc != GLOBUS_SUCCESS)
                        {
                            /* Ignore duplicates */
                            free(mime_hashentry);
                        }
                        mime_hashentry = NULL;
                    }
                    estart += token_size;
                }
                while (*estart && isspace(*estart))
                {
                    estart++;
                }
            }
        }
    }
    if (res != GLOBUS_SUCCESS)
    {
        free(mime_hashentry);
entry_alloc_fail:
        fclose(mtf);
fopen_fail:
        globus_hashtable_destroy_all(&globus_i_s3_mime_extensions, free);
    }
hashtable_init_fail:
done:
    return res;
}
/* globus_l_s3_parse_mime_types() */
// vim: ft=c:
